/*
 *  ======== clock.c ========
 *  The clock example shows how to use the ti.sysbios.knl.Clock module to
 *  create one-shot and periodic Clock Instances. Clock Instances are 
 *  essentially functions that run after a certain number of Clock ticks.
 */    

#include <xdc/std.h>
#include <xdc/runtime/System.h>

#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Clock.h>
#include <ti/sysbios/knl/Task.h>
#include <ti/sysbios/family/arm/a9/Cache.h>

Void clk0Fxn(UArg arg0);
Void task0Fxn(UArg arg0);
        

extern unsigned int __knl_load__, __knl_start__, __knl_end__;

xdc_Void copy_kernel_funcs(void)
{
    unsigned int * kl;
    unsigned int * ks;
    unsigned int * ke;

    /* relocate the .knl section */
    kl = & __knl_load__;
    ks = & __knl_start__;
    ke = & __knl_end__;
    if (kl != ks) {
        while (ks < ke) {
            *ks = *kl;
            kl++;
            ks++;
        }
    }
}

/*
 *  ======== main ========
 */
Int main()
{
    Task_Params taskParams;
    Clock_Params clkParams;

    /* Create a periodic Clock Instance with period = 5 system time units */
    Clock_Params_init(&clkParams);
    clkParams.period = 5;
    clkParams.startFlag = TRUE;
    Clock_create(clk0Fxn, 5, &clkParams, NULL);
    
    /* Create a polling task */
    Task_Params_init (&taskParams);
    taskParams.stackSize = 2048;
    taskParams.priority = 15;
    Task_create((Task_FuncPtr)task0Fxn, &taskParams, NULL);
                
    BIOS_start();    /* does not return */
    return(0);
}

/*
 *  ======== clk0Fxn =======
 */

volatile UInt32 clk0_ticks;

Void clk0Fxn(UArg arg0)
{
    UInt32 time;
    
    time = Clock_getTicks();
    //System_printf("System time in clk0Fxn = %lu\n", (ULong)time);
    clk0_ticks = time;
    Cache_wb ((Ptr) &clk0_ticks, sizeof (clk0_ticks), 0, FALSE);
}

/*
 *  ======== clk1Fxn =======
 */
Void task0Fxn(UArg arg0)
{
    UInt32 last_clk0_ticks = 0;;
    UInt32 current_clk0_ticks = 0;

    for (;;)
    {
        Task_sleep (10);
        current_clk0_ticks = clk0_ticks;
        if (current_clk0_ticks != last_clk0_ticks)
        {
            last_clk0_ticks = current_clk0_ticks;
            //System_flush ();
        }
    }
}

